; Fixes "Symbol's value as variable is void: personal-keybindings" error
(require 'bind-key)

; Disable bars
(menu-bar-mode -1)
(scroll-bar-mode -1)
(tool-bar-mode -1)

; Set font
(set-frame-font "Maple Mono NF 12" nil t)

; Set theme
(use-package catppuccin-theme :ensure t :demand t)
(load-theme 'catppuccin t)

; Set modeline
(use-package doom-modeline
  :ensure t
  :hook (after-init . doom-modeline-mode))

; Emoji support
(use-package emojify
  :hook (after-init . global-emojify-mode))

; Undo tree
(use-package undo-tree
  :demand t
  :config
  (global-undo-tree-mode)  ;; Enable undo-tree mode globally
  (setq undo-tree-history-directory-alist '(("." . "~/.emacs.d/undo"))))  ;; Directory for undo files

; Evil mode
(use-package evil
  :ensure t
  :init
  (setq evil-want-integration t) ;; This is optional since it's already set to t by default.
  (setq evil-want-keybinding nil)
   (setq evil-undo-system 'undo-tree)
  :config
  (evil-mode 1))

;; Evil collection
(use-package evil-collection
  :after evil
  :ensure t
  :config
  (evil-collection-init))

;; Evil leader
(use-package evil-leader
  :after evil
  :config
  (global-evil-leader-mode)
  (setq evil-leader/leader "<SPC>")  ;; Set the leader key to space
  (evil-leader/set-key "a" #'eglot-code-actions)
  (evil-leader/set-key "e" #'treemacs))

; Dashboard
(use-package dashboard
  :ensure t
  :config
  (dashboard-setup-startup-hook)
  (setq dashboard-startup-banner 'logo))

(defun dashboard-recentf-filter (file)
  "Add to ~recentf-exclude~ to filter recent files in dashboard buffer."
  (when (eq (get-buffer dashboard-buffer-name)
            (current-buffer))
    (string-match "treemacs-persist$" file)))

(add-to-list 'recentf-exclude 'dashboard-recentf-filter)

; Direnv
(use-package direnv
 :config
 (direnv-mode))

; Nix
(use-package nix-mode
  :mode "\\.nix\\'")

; Treesitter
(use-package tree-sitter
  :hook ((prog-mode . tree-sitter-mode)
         (prog-mode . tree-sitter-hl-mode)))

(use-package tree-sitter-langs)

(use-package treesit-auto
  :config
  (global-treesit-auto-mode)
  (dolist (lang treesit-language-source-alist)
    (let ((mode (intern (format "%s-ts-mode" (car lang)))))
      (when (fboundp mode)
        (add-to-list 'major-mode-remap-alist
                     (cons (intern (car lang)) mode))))))

; Eglot setup
(add-hook 'prog-mode-hook 'eglot-ensure)

(evil-leader/set-key
  "a" #'eglot-code-actions)

;; Function to format the buffer using Eglot
(defun my/eglot-format-buffer-on-save ()
  "Format the current buffer if Eglot is active."
  (when (bound-and-true-p eglot--managed-p)
    (eglot-format-buffer)))

;; Add the function to the save-hook
(add-hook 'before-save-hook #'my/eglot-format-buffer-on-save)

; Flycheck
(use-package flycheck
  :ensure t
  :config
  (add-hook 'after-init-hook #'global-flycheck-mode))

; Sideline
(use-package sideline-flycheck
  :hook (flycheck-mode . sideline-mode)
  :init (setq sideline-backends-right '(sideline-flycheck)
              sideline-delay 0.0))

(use-package sideline-flycheck
  :hook (flycheck-mode . sideline-flycheck-setup)
  :init (setq sideline-flycheck-display-mode 'line))

; Eglot
(use-package sideline-eglot
  :init
  (setq sideline-backends-right '(sideline-eglot)))

(use-package eglot :hook (eglot-mode . sideline-mode))  ; enable it when eglot is on

; Centaur tabs
(use-package centaur-tabs
  :demand t
  :config
  (centaur-tabs-mode t)
  (setq centaur-tabs-style "wave")
  :bind
  ("C-<prior>" . centaur-tabs-backward)
  ("C-<next>" . centaur-tabs-forward))
